import numpy as np
from PIL import Image

from lentille.base_embedder import BaseEmbedder
from lentille.utils import resnet50_preprocess


class Resnet50Embedder(BaseEmbedder):
    """
    Always instantiate using: Resnet50Embedder.from_file(model_file.onnx)
    Models can be found here: https://github.com/onnx/models/tree/main/vision/classification/resnet

    """

    def _preprocess(self, image):
        return resnet50_preprocess.preprocess(image)

    def classify(self, image_location: str) -> np.array:
        image = Image.open(image_location)
        preprocessed_image = self._preprocess(image)

        # Magic structure to be able to speak to onnx?
        input_name = self.session.get_inputs()[0].name
        input_data = {input_name: preprocessed_image}

        # Label overview: https://files.fast.ai/models/imagenet_class_index.json
        return self.session.run([], input_data)[0]



from abc import ABC
import onnxruntime


class BaseEmbedder(ABC):
    """
    Base Embedder class. For now this class is instantiated using an onnx session, this might be abstracted higher or lower later on.
    """

    def __init__(self, session):
        self.session = session

    @classmethod
    def from_file(cls, file):
        return cls(onnxruntime.InferenceSession(file))



import numpy as np
from PIL import Image


def preprocess(image: Image) -> np.array:
    """
    Resnet requires images to be of certain structure. This function performs the various transformations
    """
    image = image.convert("RGB")  # convert to rbg so normalization step works

    # resize so that the shorter side is 256, maintaining aspect ratio
    def image_resize(image, min_len):
        # image = Image.fromarray(image)
        ratio = float(min_len) / min(image.size[0], image.size[1])
        if image.size[0] > image.size[1]:
            new_size = (int(round(ratio * image.size[0])), min_len)
        else:
            new_size = (min_len, int(round(ratio * image.size[1])))
        image = image.resize(new_size, Image.BILINEAR)
        return np.array(image)

    image = image_resize(image, 256)

    # Crop centered window 224x224
    def crop_center(image, crop_w, crop_h):
        h, w, c = image.shape
        start_x = w // 2 - crop_w // 2
        start_y = h // 2 - crop_h // 2
        return image[start_y : start_y + crop_h, start_x : start_x + crop_w, :]

    image = crop_center(image, 224, 224)

    # transpose
    image = image.transpose(2, 0, 1)

    # convert the input data into the float32 input
    img_data = image.astype("float32")

    # normalize
    mean_vec = np.array([0.485, 0.456, 0.406])
    stddev_vec = np.array([0.229, 0.224, 0.225])
    norm_img_data = np.zeros(img_data.shape).astype("float32")
    for i in range(img_data.shape[0]):
        norm_img_data[i, :, :] = (img_data[i, :, :] / 255 - mean_vec[i]) / stddev_vec[i]

    # add batch channel
    norm_img_data = norm_img_data.reshape(1, 3, 224, 224).astype("float32")

    return norm_img_data



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['lentille', 'lentille.embedders.vision', 'lentille.utils']

package_data = \
{'': ['*']}

install_requires = \
['numpy>=1.24.3,<2.0.0', 'onnxruntime>=1.14.1,<2.0.0', 'pillow>=9.5.0,<10.0.0']

setup_kwargs = {
    'name': 'lentille',
    'version': '0.2.0',
    'description': '',
    'long_description': '# Lentille\nLibrary that wraps various embedders. For now it only wraps a Resnet50 model.\n\n## Installation\n``` bash\npip install lentille\n```\n\n## Example usage\n``` python\nfrom lentille import Resnet50Embedder\n\nembedder = Resnet50Embedder.from_file(\n    "resnet50-v1-12-int8.onnx" # ONNX model from: https://github.com/onnx/models/tree/main/vision/classification/resnet\n)\n\nprint(embedder.classify("cat.jpg"))\n```\n',
    'author': 'Alex',
    'author_email': '46456279+the-alex-b@users.noreply.github.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)
